//------------------------------------------------------------------------------
//
// Copyright (c) 2008-2015 IAR Systems
//
// Licensed under the Apache License, Version 2.0 (the "License")
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// $Revision: 38952 $
//
//------------------------------------------------------------------------------

#include "flash_loader.h"
#include "flash_loader_extra.h"

#include "flash_loader_ram.h"
#include <intrinsics.h>

void strcopy(char *to, char* from);
#define UNUSED(x) (void)(x)

void strcopy(char *to, char* from)
{
  while (*to++ = *from++) ;
}

#define delay(mks) delay_hw((int)((mks)*1.0)) // derating

//#define delay_sw(mks)  {for(register int i=(mks); i>=1; i--) __asm__ volatile ("nop;nop;nop;nop;nop;nop;nop;nop;");}

static inline void __attribute__((always_inline))
delay_hw (register uint32_t mks) {
	register uint32_t t;
	t = __read_csr(_CSR_MCYCLE) + mks*(CORE_FREQ/1000000);
	while ( (int32_t)(__read_csr(_CSR_MCYCLE) - t) < 0 ) {}
}

uint32_t Erase(uint32_t MassSelector, uint32_t block_start, uint32_t block_size)
{
  register uint32_t BankSelector = (uint32_t)block_start<FLASH_BASE;
//  register uint32_t MassSelector = 0;
  register uint32_t Current_Delay;
  register uint32_t chip;
  UNUSED(block_size);

  EEPROM->KEY = EEPROM_Access_Enable;
  Current_Delay = EEPROM->CMD & (1<<3);
  EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE;
  EEPROM->CTRL=0;
  for(chip=0;chip<2;chip++) {
    EEPROM->ADR = ((uint32_t)block_start>>1)+(BankSelector? 0x00002000 : 0x00040000)*chip;
    EEPROM->DI = 0;
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_ERASE_S | MassSelector << EEPROM_CMD_CHIP_S; //CE=1, ERASE=1,CHIP=MassSelector, NVR=BankSelector,WE=0

    if (MassSelector) {
      delay(MLDR215_FLASH_TNVS_CHIP_ERASE); /* Wait for 80us  Time Tnvs Chip Erase*/
    } else {
      delay(MLDR215_FLASH_TNVS_SECT_ERASE); /* Wait for 20us Time Tnvs Sector Erase*/
    }

    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S |  1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_ERASE_S | MassSelector << EEPROM_CMD_CHIP_S | 1 << EEPROM_CMD_WE_S; //CE=1, ERASE=1,CHIP=MassSelector,NVR=BankSelector, WE=1

    if (MassSelector) {
      delay(MLDR215_FLASH_TERASE_MASS); /* Wait for 40ms Terase   Chip Erase*/
    } else {
      delay(MLDR215_FLASH_TERASE_SECT); /* Wait for 3ms Terase Sector Erase*/
    }

    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S |  1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_ERASE_S | MassSelector << EEPROM_CMD_CHIP_S ; //CE=1, ERASE=1,CHIP=MassSelector,NVR=BankSelector, WE=0

    if (MassSelector) {
      delay(MLDR215_FLASH_TRCV_CHIP_ERASE); /* Wait for 200us Trcv Chip Erase */
    } else {
      delay(MLDR215_FLASH_TRCV_SECT_ERASE); /* Wait for 50us Trcv Sector Erase */
    }

    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE |  1 << EEPROM_CMD_CE_S ; //CE=1, ERASE=0,CHIP=0, WE=0
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE;

    delay(10); /* Wait for 10us Time Trw2  */
  }

  EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_DISABLE | EEPROM_CMD_TMEN_DISABLE;
  EEPROM->KEY = 0x0;

  return 0;
}


uint32_t FlashInit(void *base_of_flash, uint32_t image_size,
                   uint32_t link_address, uint32_t flags,
                   int argc, char const *argv[])
{
//  strcopy(LAYOUT_OVERRIDE_BUFFER,"2 0x100,7 0x200,7 0x1000");
//  SET_BUFSIZE_OVERRIDE(&FlashBufferEnd-&FlashBufferStart+1);
//  SET_PAGESIZE_OVERRIDE(128);
//  return OVERRIDE_LAYOUT | OVERRIDE_BUFEND | OVERRIDE_PAGESIZE;
  if (flags & FLAG_ERASE_ONLY)
  {
//      strcopy(ERROR_MESSAGE_BUFFER, "Not implemented");
//      return RESULT_ERROR_WITH_MSG;

  // perform full erase
  return Erase(EEPROM_Mass_Erase, 0, 0);
  }

// clock enabled in .mac file  MDR_RCC->PER2_CLOCK |= RCC_PER2_CLOCK_EEPROMEN;

  return RESULT_OK;
}

uint32_t ConvertAddr (uint32_t addr){
  uint32_t cell_addr;
  uint32_t word_addr;
  uint32_t new_addr;

  cell_addr=addr/8;
  word_addr=addr/4;

  if (word_addr%2==0)  //from flash1 ip
  {
    new_addr=cell_addr*4;
  } else { //frm flash2 ip
    new_addr=cell_addr*4 + (addr<FLASH_BASE? 0x00002000 : 0x00040000);
  }

return new_addr;
}

uint32_t FlashWrite(void *block_start,
                    uint32_t offset_into_block,
                    uint32_t count,
                    char const *buffer)
{
  register uint32_t address;
  register uint32_t BankSelector = (uint32_t)block_start<FLASH_BASE;
  register uint32_t Current_Delay;

  address = (uint32_t)block_start + offset_into_block;

  EEPROM->KEY = EEPROM_Access_Enable;
  Current_Delay = EEPROM->CMD & (1<<3);
  EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE;
  EEPROM->CTRL=0;
  while (count > 0) {
    EEPROM->ADR = ConvertAddr(address);
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_PROG_S ; //CE=1, PROG=1, NVR=BankSelector,WE=0,PROG2=0
    delay(MLDR215_FLASH_TNVS_PROG); // Tnvs Programm
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_PROG_S | 1 << EEPROM_CMD_WE_S ; //CE=1, PROG=1, NVR=BankSelector,WE=1,PROG2=0
    delay(MLDR215_FLASH_TPGS); // Tpgs

    for(int i = address%4; i < 4; i++) {
      EEPROM->CTRL=(1<<i)<<EEPROM_CTRL_BYTE_S;
      EEPROM->DI = (uint32_t)*buffer++ << (i*8);
      NOP; NOP; NOP; NOP; // Tads
      EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_PROG_S | 1 << EEPROM_CMD_WE_S | 1 << EEPROM_CMD_PROG2_S  ; //CE=1, PROG=1, NVR=BankSelector,WE=1,PROG2=1
      delay(MLDR215_FLASH_TPROG); //Tprog min 5us max 6.5us Tprog
      EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_PROG_S | 1 << EEPROM_CMD_WE_S  ; //CE=1, PROG=1, NVR=BankSelector,WE=1,PROG2=0
      NOP; NOP; NOP; NOP; // TAdh
      address++;
      if (0 == --count)
        break;
    }
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S | 1 << EEPROM_CMD_CE_S | 1 << EEPROM_CMD_PROG_S ; //CE=1, PROG=1, NVR=BankSelector,WE=0,PROG2=0
    delay(MLDR215_FLASH_TRCV_PROG); // Trcv Prog
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE | BankSelector<<EEPROM_CMD_NVR_S |1 << EEPROM_CMD_CE_S ; //CE=1, PROG=0, NVR=BankSelector,WE=0,PROG2=0
    EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_ENABLE | EEPROM_CMD_TMEN_DISABLE;
    delay(1); // Wait for 1us Trw
    }
  EEPROM->CMD = Current_Delay | EEPROM_CMD_CON_DISABLE | EEPROM_CMD_TMEN_DISABLE;
  EEPROM->CTRL= 0x0;
  EEPROM->KEY = 0x0;

  return 0;
}

uint32_t FlashErase(void *block_start, uint32_t block_size)
{
  //char *p = (char*)block_start;
  //while (block_size--)
  //{
  //  *p++ = 0;
  //}

//      while (block_size > 0) {
//            block_start = (void *) ((unsigned)block_start + 0x1000);
//            block_size -= 0x1000;
//            }

  return Erase(EEPROM_Sector_Erase, (uint32_t)block_start, block_size);
}

OPTIONAL_CHECKSUM
uint32_t FlashChecksum(void const *begin, uint32_t count)
{
  return Crc16((uint8_t const *)begin, count);
}

OPTIONAL_SIGNOFF
uint32_t FlashSignoff(void)
{
  //MDR_RCC->PER2_CLOCK &=~RCC_PER2_CLOCK_EEPROMEN;

  return 0;
}

#ifdef DEBUG
const char Placeholder[] = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.";

void main(int argc, char const *argv[])
{
  //_SP = 0x4000FFF0;
  RST_CLK->PER2_CLOCK |= RCC_PER2_CLOCK_EEPROMEN;
  BKP->BKP_WPR = 0x8555AAA1;
  //*** Set HSI to 8MHz
//  BKP->BKP_CLK |= HSI_TRIM_TO8MHZ;

  FlashInit(0, 0, 0, 0, argc, argv);
  NOP;
  FlashErase((void *)0x10000000, 0);
//  FlashErase((void *)0x10000100, 0);
//  FlashErase((void *)0x10000200, 0);
  FlashErase((void *)0x10000400, 0);
  NOP;
  FlashWrite((void *)0x10000000, 0, sizeof(Placeholder), Placeholder);
  FlashWrite((void *)0x10000100, 0, sizeof(Placeholder), Placeholder);
  FlashWrite((void *)0x10000200, 0, sizeof(Placeholder), Placeholder);
  FlashWrite((void *)0x10000400, 0, sizeof(Placeholder), Placeholder);
  NOP;
  FlashErase((void *)0x10000100, 0);
  FlashErase((void *)0x10000200, 0);
  FlashErase((void *)0x10000400, 0);
  NOP;

  FlashWrite((void *)0x10000300, 0, 1, Placeholder);
  NOP;
  FlashWrite((void *)0x10000310, 0, 2, Placeholder);
  NOP;
  FlashWrite((void *)0x10000320, 0, 3, Placeholder);
  NOP;
  FlashWrite((void *)0x10000330, 0, 4, Placeholder);
  NOP;
  FlashWrite((void *)0x10000341, 0, 1, Placeholder);
  NOP;
  FlashWrite((void *)0x10000352, 0, 1, Placeholder);
  NOP;
  FlashWrite((void *)0x10000363, 0, 1, Placeholder);
  NOP;
  FlashWrite((void *)0x10000371, 0, 5, Placeholder);
  NOP;
  FlashWrite((void *)0x10000371, 0, 5, Placeholder+1);
  NOP;

  FlashErase((void *)0x00020000, 0);
  NOP;
  FlashWrite((void *)0x00020000, 0, sizeof(Placeholder), Placeholder);
  NOP;
  FlashErase((void *)0x00020800, 0);
  NOP;

//  Erase(EEPROM_Mass_Erase, 0, 0);
  FlashInit(0, 0, 0, FLAG_ERASE_ONLY, argc, argv);
  NOP;
}
#endif


