//------------------------------------------------------------------------------
//
// Copyright (c) 2008-2015 IAR Systems
//
// Licensed under the Apache License, Version 2.0 (the "License")
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// $Revision: 38952 $
//
//------------------------------------------------------------------------------

#include "flash_loader.h"
#include "flash_loader_extra.h"

#include "flash_loader_ram.h"
#include <intrinsics.h>

#define CORE_FREQ 8000000ul

//void strcopy(char *to, char* from);

#define delay(mks) delay_hw((int)((mks)*1.0)) // derating

//#define delay_sw(mks)  {for(register int i=(mks); i>=1; i--) __asm__ volatile ("nop;nop;nop;nop;nop;nop;nop;nop;");}

static inline void __attribute__((always_inline))
delay_hw (register uint32_t mks) {
	register uint32_t t;
	t = __read_csr(_CSR_MCYCLE) + mks*(CORE_FREQ/1000000);
	while ( (int32_t)(__read_csr(_CSR_MCYCLE) - t) < 0 ) {}
}


uint32_t FlashInit(void *base_of_flash, uint32_t image_size,
                   uint32_t link_address, uint32_t flags,
                   int argc, char const *argv[])
{
//  strcopy(LAYOUT_OVERRIDE_BUFFER,"2 0x100,7 0x200,7 0x1000");
//  SET_BUFSIZE_OVERRIDE(&FlashBufferEnd-&FlashBufferStart+1);
//  SET_PAGESIZE_OVERRIDE(128);
//  return OVERRIDE_LAYOUT | OVERRIDE_BUFEND | OVERRIDE_PAGESIZE;
  if (flags & FLAG_ERASE_ONLY)
  {
    // perform full erase
    
    register uint32_t mem_info;
    register uint32_t flash_cmd;

    mem_info = 0;

    EEPROM_CNTR->KEY = 0x8AAA5551; 
    flash_cmd = EEPROM_CNTR->CMD;

    /* Switch on register access */
    flash_cmd = (flash_cmd & FLASH_DELAY_MASK) | FLASH_CON | FLASH_TMR;
    if (mem_info)
            flash_cmd |= FLASH_IFREN;
    EEPROM_CNTR->CMD = flash_cmd;
    for(uint32_t chip=0;chip<=1;chip++)
        {
        EEPROM_CNTR->ADR = chip<<18;

        flash_cmd |= FLASH_XE | FLASH_MAS1 | FLASH_ERASE;
        EEPROM_CNTR->CMD = flash_cmd;
        delay(FLASH_Tnvs);
        flash_cmd |= FLASH_NVSTR;
        EEPROM_CNTR->CMD = flash_cmd;
        delay(FLASH_Tme);
        flash_cmd &= ~FLASH_ERASE;
        EEPROM_CNTR->CMD = flash_cmd;
        delay(FLASH_Tnvh1);
        flash_cmd &= ~(FLASH_XE | FLASH_MAS1 | FLASH_NVSTR);
        EEPROM_CNTR->CMD = flash_cmd;
        delay(FLASH_Trcv);
        }
    flash_cmd &= FLASH_DELAY_MASK;
    EEPROM_CNTR->CMD = flash_cmd;        
    EEPROM_CNTR->KEY = 0;
    
    
    return RESULT_ERASE_DONE;
  }

// clock enabled im .mac file  MDR_RCC->PER2_CLOCK |= RCC_PER2_CLOCK_EEPROMEN;
    
  return RESULT_OK;
}

#ifdef DEBUG
#include "string.h"
#include "assert.h"
const char Placeholder[] = "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.";

int memcomp(const void* s1, const void* s2, size_t n)
{
  unsigned char *p1 = (unsigned char *)s1;
  unsigned char *p2 = (unsigned char *)s2;
  unsigned char u1, u2;

  for ( ; n-- ; ) {
    u1 = *p1++;
    u2 = *p2++;
    if ( u1 != u2) {
      return (u1-u2);
    }
  }
  return 0;  
}

int memcheck(const void* s1, size_t n)
{
  unsigned char *p1 = (unsigned char *)s1;

  for ( ; n-- ; ) {
    if ( 0xFF != *p1++) {
      return (-1);
    }
  }
  return 0;  
}

void main(int argc, char const *argv[])
{
  volatile uint32_t r;
  volatile uint32_t ERROR=0;

  MDR_RCC->PER2_CLOCK |= RCC_PER2_CLOCK_EEPROMEN;
//  MDR_RCC->CPU_CLOCK &=~(3<<8);
  MDR_RCC->CPU_CLOCK = (1<<8) |(0<<4) |(0<<2) |(0<<0);
  NOP;
  r = FlashInit ((void *)0x10000000, 0, 0, 0, argc, argv);
  if (r!=RESULT_OK)
    ERROR++;    
  r = FlashErase((void *)0x00020000, 0x01000);
  if (  memcheck((void *)0x00020000, 0x01000))
    ERROR++;    
  r = FlashWrite((void *)0x00020000,       0, sizeof(Placeholder), Placeholder);
  if (r!=RESULT_OK)
    ERROR++;    
  if (   memcomp((void *)0x00020000,         Placeholder, sizeof(Placeholder)))
    ERROR++;    

  r = FlashErase((void *)0x00021000, 0x01000);
  if (  memcheck((void *)0x00021000, 0x01000))
    ERROR++;    
  r = FlashWrite((void *)0x00021000,       0, sizeof(Placeholder), Placeholder);
  if (r!=RESULT_OK)
    ERROR++;    
  if (   memcomp((void *)0x00021000,         Placeholder, sizeof(Placeholder)))
    ERROR++;    

  r = FlashErase((void *)0x00022000, 0x01000);
  if (  memcheck((void *)0x00022000, 0x01000))
    ERROR++;    
  r = FlashWrite((void *)0x00022000,       0, sizeof(Placeholder), Placeholder);
  if (r!=RESULT_OK)
    ERROR++;    
  if (   memcomp((void *)0x00022000,         Placeholder, sizeof(Placeholder)))
    ERROR++;    

  r = FlashErase((void *)0x00023000, 0x01000);
  if (  memcheck((void *)0x00023000, 0x01000))
    ERROR++;    
  r = FlashWrite((void *)0x00023000,       0, sizeof(Placeholder), Placeholder);
  if (r!=RESULT_OK)
    ERROR++;    
  if (   memcomp((void *)0x00023000,         Placeholder, sizeof(Placeholder)))
    ERROR++;    

  NOP;
  r = FlashErase((void *)0x10000000, 0x01000);
//  if ((*(uint32_t *)0x10000000!=0xFFFFFFFF) || (*(uint32_t *)0x10000FFC!=0xFFFFFFFF))
  if (  memcheck((void *)0x10000000, 0x01000))
    ERROR++;    
  if (r!=RESULT_OK)
    ERROR++;    
  r = FlashWrite((void *)0x10000000,       0, sizeof(Placeholder), Placeholder);
  if (r!=RESULT_OK)
    ERROR++;    
  if (   memcomp((void *)0x10000000,         Placeholder, sizeof(Placeholder)))
    ERROR++;    
  NOP;
  r = FlashErase((void *)0x10001000, 0x01000);
  if (  memcheck((void *)0x10001000, 0x01000))
    ERROR++;    
  r = FlashWrite((void *)0x10001000,       0, sizeof(Placeholder)-4, Placeholder+4);
  if (   memcomp((void *)0x10001000,         Placeholder+4, sizeof(Placeholder)-4))
    ERROR++;    
  NOP;
  r = FlashErase((void *)0x10040000, 0x02000); // IP2
  if (  memcheck((void *)0x10040000, 0x02000))
    ERROR++;    
  r = FlashWrite((void *)0x10040000,       4, sizeof(Placeholder)-8, Placeholder+8);
  if (   memcomp((void *)0x10040004,          Placeholder+8, sizeof(Placeholder)-8))
    ERROR++;    
  r = FlashWrite((void *)0x10040000,       4, sizeof(Placeholder)-1, Placeholder+1);
  if (r!=RESULT_ERROR)
    ERROR++;    
  NOP;
  r = FlashInit ((void *)0x10000000, 0, 0, FLAG_ERASE_ONLY, argc, argv);
  if (r!=RESULT_ERASE_DONE)
    ERROR++;    
  if (  memcheck((void *)0x10000000, 0x40000))
    ERROR++;    
  if (  memcheck((void *)0x10040000, 0x40000))
    ERROR++;    
  NOP;
  (void)ERROR;
}
#endif

uint32_t FlashWrite(void *block_start,
                    uint32_t offset_into_block,
                    uint32_t count,
                    char const *buffer)
{
//  char *to = (char*)block_start + offset_into_block;
//  while (count--)
//  {
//    *to++ = *buffer++;
//  }
  register uint32_t address;
  register uint32_t *buf;
  register uint32_t mem_info;
  register uint32_t flash_cmd;

  address = (uint32_t)block_start + offset_into_block;
  if (address&3)
    return RESULT_ERROR;
  if ((uint32_t)buffer&3)
    return RESULT_ERROR;
  buf = (uint32_t *)buffer;
  count = (count + 3) & ~3;  // Adjust size 
  
  mem_info = (uint32_t)block_start < 0x10000000;

  EEPROM_CNTR->KEY = 0x8AAA5551; // *(uint32_t *)(FLASH_KEY) = KEY;
  flash_cmd = EEPROM_CNTR->CMD;

  // Switch on register access 
  flash_cmd = (flash_cmd & FLASH_DELAY_MASK) | FLASH_CON | FLASH_TMR;
  if (mem_info)
          flash_cmd |= FLASH_IFREN;
  EEPROM_CNTR->CMD = flash_cmd;

  while (count > 0) {
    EEPROM_CNTR->ADR = address;
    EEPROM_CNTR->DI = *buf;

    flash_cmd |= FLASH_XE | FLASH_PROG;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Tnvs);
    flash_cmd |= FLASH_NVSTR;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Tpgs);
    flash_cmd |= FLASH_YE;
    EEPROM_CNTR->CMD = flash_cmd;

    delay(FLASH_Tprog);

    flash_cmd &= ~FLASH_YE;
    EEPROM_CNTR->CMD = flash_cmd;

    flash_cmd &= ~FLASH_PROG;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Tnvh);
    flash_cmd &= ~(FLASH_XE | FLASH_NVSTR);
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Trcv);

    buf += 1;
    address += 4;
    count -= 4;
  }

  flash_cmd &= FLASH_DELAY_MASK;
  EEPROM_CNTR->CMD = flash_cmd;
  EEPROM_CNTR->KEY = 0;  

  return RESULT_OK;
}

uint32_t FlashErase(void *block_start, uint32_t block_size)
{
  //char *p = (char*)block_start;
  //while (block_size--)
  //{
  //  *p++ = 0;
  //}

  register uint32_t mem_info;
  register uint32_t flash_cmd;

  mem_info =  (uint32_t)block_start < 0x10000000;

  EEPROM_CNTR->KEY = 0x8AAA5551; 
  flash_cmd = EEPROM_CNTR->CMD;

  /* Switch on register access */
  flash_cmd = (flash_cmd & FLASH_DELAY_MASK) | FLASH_CON | FLASH_TMR;
  if (mem_info)
          flash_cmd |= FLASH_IFREN;
  EEPROM_CNTR->CMD = flash_cmd;

  while (block_size > 0) {
    EEPROM_CNTR->ADR = (uint32_t)block_start;

    flash_cmd |= FLASH_XE | FLASH_ERASE;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Tnvs); // Tnvs
    flash_cmd |= FLASH_NVSTR;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Terase); // Terase
    flash_cmd &= ~FLASH_ERASE;
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Tnvh); // Tnvh
    flash_cmd &= ~(FLASH_XE | FLASH_NVSTR);
    EEPROM_CNTR->CMD = flash_cmd;
    delay(FLASH_Trcv); // Trcv
    
    block_start = (void *) ((uint32_t)block_start + PAGE_SIZE);
    block_size -= PAGE_SIZE;
    }
  
  flash_cmd &= FLASH_DELAY_MASK;
  EEPROM_CNTR->CMD = flash_cmd;
  EEPROM_CNTR->KEY = 0;
  return RESULT_OK;
}

OPTIONAL_CHECKSUM
uint32_t FlashChecksum(void const *begin, uint32_t count)
{
  return Crc16((uint8_t const *)begin, count);
}

OPTIONAL_SIGNOFF
uint32_t FlashSignoff(void)
{
  //MDR_RCC->PER2_CLOCK &=~RCC_PER2_CLOCK_EEPROMEN;

  return 0;
}

//void strcopy(char *to, char* from)
//{
//  while (*to++ = *from++) ;
//}




